@@ -39,6 +39,7 @@
 #include<GyotoAstrobj.h>
 #include<GyotoThinDisk.h>
 #include<GyotoPageThorneDisk.h>
+#include<GyotoDirectionalDisk.h>
 #include<GyotoPatternDisk.h>
 #include<GyotoPatternDiskBB.h>
 #include<GyotoDynamicalDisk.h>
@@ -51,9 +52,7 @@
 <B><CENTER> General relativitY Orbit Tracer of Observatoire de Paris</CENTER></B>
  *
  *
- <A HREF="https://github.com/gyoto/Gyoto"><B>Download</B></A> | <A
- HREF="http://github.com/gyoto/Gyoto/blob/master/INSTALL"><B>Installation
- instructions</B></A> | \ref conditions_sec
+ \ref download_sec | \ref manual_sec | \ref conditions_sec
  *
  * Gyoto aims at providing a framework for computing orbits and
  * ray-traced images in General relativity. It consists of a library
@@ -72,8 +71,26 @@
  * A graphical user interface for tracing stellar orbits is provided
  * with the Yorick plug-in (see \ref yorick_plugin_page).
  *
+ *
  * \image html gyotoy_screenshot.png
  *
+ * \section download_sec Downloading and installing
+ *
+ * Gyoto is version-controlled by the 
+ * <A HREF="https://github.com/gyoto/Gyoto"><B>git</B></A> software.
+ * 
+ * The easiest way to acces it is by typing on a command line:
+ *
+ * <I><B>git clone git://github.com/gyoto/Gyoto.git</B></I> 
+ *
+ * More information on installing Gyoto is available 
+ * <A HREF="http://github.com/gyoto/Gyoto/blob/master/INSTALL"><B>here</B></A>
+ * as well as in the user manual below.
+ *
+ * \section manual_sec User manual
+ *
+ * The user manual is available <A HREF="GyotoManual.pdf"><B>here</B></A>.
+ *
  * \section conditions_sec Conditions for use
  * 
  * We request that use of Gyoto in scientific publications be properly
