@@ -74,7 +74,9 @@ namespace Gyoto{
     template<typename T> SmartPointer<Astrobj::Generic> Subcontractor
       (FactoryMessenger* fmp) {
       SmartPointer<T> ao = new T();
+#ifdef GYOTO_USE_XERCES
       ao -> setParameters(fmp);
+#endif
       return ao;
     }
     ///< A template for Subcontractor_t functions
@@ -90,10 +92,9 @@ namespace Gyoto{
      * \return pointer to the corresponding subcontractor.
      */
     Gyoto::Astrobj::Subcontractor_t* getSubcontractor(std::string name,
-						      int errmode = 1);
+						      int errmode = 0);
     ///< Query the Astrobj register
 
-#if defined GYOTO_USE_XERCES
     /**
      * Use the Astrobj::initRegister() once in your program to
      * initiliaze it, the Astrobj::Register() function to fill it, and
@@ -121,7 +122,6 @@ namespace Gyoto{
      */
     void Register(std::string name, Gyoto::Astrobj::Subcontractor_t* scp);
     ///< Make an Astrobj kind known to the Factory
-#endif
   }
 }
 
@@ -201,21 +201,21 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
   /**
    * Maximum distance from the center of the coordinate system at
    * which a photon may hit the object.  Child classes may choose to
-   * update rmax at all time or to use it to cache the value, for
-   * instance when getRmax() is called. External classes (Photons in
-   * particular) must use getRmax() to access this information.
+   * update rMax at all time or to use it to cache the value, for
+   * instance when rMax() is called. External classes (Photons in
+   * particular) must use rMax() to access this information.
    *
-   * rmax_set_==1 means that rmax_ was set using setRmax() or the
-   * constructor. In this case, getRmax() must always return this
+   * #rmax_set_==1 means that #rmax_ was set using rMax() or the
+   * constructor. In this case, rMax() must always return this
    * value, not recompute it.
    *
-   * rmax_ is in geometrical units.
+   * #rmax_ is in geometrical units.
    */                                         
   double rmax_; ///< Maximum distance to the center of the coordinate system [geometrical units]
 
   /**
-   * rmax_set_==1 means that rmax_ was set using setRmax() or the
-   * constructor. In this case, getRmax() must always return this
+   * #rmax_set_==1 means that #rmax_ was set using rMax(double r) or the
+   * constructor. In this case, rMax() must always return this
    * value, not recompute it.
    *
    * Use unsetRmax() to reset rmax_set_ to 0.
@@ -227,7 +227,7 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    * The kind should match the name of the class, e.g. "Star" for a
    * Gyoto::Star.
    */
-  const std::string kind_; ///< Kind of object (e.g. "Star"...)
+  std::string kind_; ///< Kind of object (e.g. "Star"...)
 
   int flag_radtransf_; ///< 1 if radiative transfer inside Astrobj, else 0
 
@@ -235,17 +235,17 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
   // -------------------------
  public:
   /**
-   *  kind_ =  "Default", rmax_ = 0., rmax_set_ = 0.
+   *  #kind_ =  "Default", #rmax_ = 0., #rmax_set_ = 0.
    */
   Generic(); ///< Default constructor.
 
   /**
-   *  kind_ =  "Default", rmax_ = radmax, rmax_set_ = 1.
+   *  #kind_ =  "Default", #rmax_ = radmax, #rmax_set_ = 1.
    */
   Generic(double radmax); ///< Set rmax in constructor.
 
   /**
-   *  kind_ =  kind, rmax_ = 0., rmax_set_ = 0.
+   *  #kind_ =  kind, #rmax_ = 0., #rmax_set_ = 0.
    */
   Generic(std::string kind); ///< Set kind in constructor.
 
@@ -278,14 +278,14 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
   // ---------
  public:
   /**
-   * \brief Get the Metric Generic::gg_
+   * \brief Get the Metric #gg_
    */
-  virtual SmartPointer<Metric::Generic> getMetric() const;
+  virtual SmartPointer<Metric::Generic> metric() const;
 
   /**
-   * \brief Set the Metric Generic::gg_
+   * \brief Set the Metric #gg_
    */
-  virtual void setMetric(SmartPointer<Metric::Generic>) ;
+  virtual void metric(SmartPointer<Metric::Generic>) ;
 
   /**
    *  Get maximal distance from center of coordinate system at which a
@@ -293,44 +293,51 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    *  
    *  Child classes may use the rmax_ member to cache this value.
    *
-   *  It can also be set using setRmax(). If setRmax has been used
-   *  to set rmax_, getRmax() must not recompute it.
+   *  It can also be set using rmax(). If rmax has been used
+   *  to set rmax_, rMax() must not recompute it.
    *
    *  \return rmax_ in geometrical units
    */
-  virtual double getRmax(); ///< Get maximal distance from center of coordinate system
+  virtual double rMax(); ///< Get maximal distance from center of coordinate system
 
   /**
-   *  Call getRmax() and convert result to unit.
+   *  Call rMax() and convert result to unit.
    *
    *  \param unit string
    *  \return double rmax converted to unit
    */
-  virtual double getRmax(std::string unit); ///< Get rmax_ is specified unit
+  virtual double rMax(std::string unit); ///< Get rmax_ is specified unit
 
-  const std::string getKind() const; ///< Get the kind of the Astrobj (e.g. "Star")
+  /// Get max step constraint for adaptive integration
+  /**
+   * \param[in] coord position
+   * \return max step to find this object reliably
+   */
+  virtual double deltaMax(double coord[8]);
+
+  const std::string kind() const; ///< Get the kind of the Astrobj (e.g. "Star")
 
   /**
    *  Set maximal distance from center of coordinate system at which a
    *  Photon may hit the object.
    *  
-   *  Side effect: set rmax_set_ to 1.
-   *  \param val new rmax_ in geometrical units.
+   *  Side effect: set #rmax_set_ to 1.
+   *  \param val new #rmax_ in geometrical units.
    */
-  virtual void setRmax(double val); ///< Set maximal distance from center of coordinate system
+  virtual void rMax(double val); ///< Set maximal distance from center of coordinate system
 
   /**
-   *  Call Generic::setRmax(double val) after converting val from unit
+   *  Call Generic::rMax(double val) after converting val from unit
    *  to geometrical units.
    *
-   *  \param val rmax_ expressed in unit "unit";
+   *  \param val #rmax_ expressed in unit "unit";
    *  \param unit string...
    */
-  virtual void setRmax(double val, std::string unit); ///< Set maximal distance from center of coordinate system
+  virtual void rMax(double val, std::string unit); ///< Set maximal distance from center of coordinate system
 
   /**
-   * getRmax() will then be free to recompute rmax_. Astrobjs
-   * which cache rmax_ may need to update it when unsetRmax() is
+   * rmax() will then be free to recompute rmax_. Astrobjs
+   * which cache rmax_ may need to update it when unrmax() is
    * called.
    */
   virtual void unsetRmax() ; ///< Set rmax_set_ to 0.
@@ -340,12 +347,12 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    * i.e. the object is to be considered optically thin.
    * \param flag: 1 if optically thin, 0 if optically thick.
    */
-  void setFlag_radtransf(int flag);
+  void opticallyThin(int flag);
   ///< Set whether the object is optically thin.
   /**
-   * See setFlag_radtransf(int flag).
+   * See opticallyThin(int flag).
    */
-  int getFlag_radtransf() const ;
+  int opticallyThin() const ;
   ///< Query whether object is optically thin.
 
   /**
@@ -672,7 +679,7 @@ class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
    *  Spectra elements are separated by offset doubles in memory. In
    *  other words, the ith spectral element is spectrum[i*offset].
    */
-  int offset; ///< How to jump from one spectral element to the next
+  ptrdiff_t offset; ///< How to jump from one spectral element to the next
 
   /**
    * Coordinates of the object and photon at impact
@@ -746,18 +753,27 @@ class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
    * impactcoords which is incremented by 16.
    */
   Properties operator++();
+
+  /**
+   * \brief Increment pointers by offset
+   *
+   * All valid pointers are incremented by offset (sizeof(double)), excepted
+   * impactcoords which is incremented by 16*offset.
+   */
+  Properties operator+=(ptrdiff_t offset);
+
 # ifdef HAVE_UDUNITS
-  void setIntensityConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
+  void intensityConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
   ///< Set Properties::intentity_converter_
-  void setIntensityConverter(std::string);
+  void intensityConverter(std::string);
   ///< Set Properties::intentity_converter_
-  void setSpectrumConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
+  void spectrumConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
   ///< Set Properties::spectrum_converter_
-  void setSpectrumConverter(std::string);
+  void spectrumConverter(std::string);
   ///< Set Properties::spectrum_converter_
-  void setBinSpectrumConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
+  void binSpectrumConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
   ///< Set Properties::binspectrum_converter_
-  void setBinSpectrumConverter(std::string);
+  void binSpectrumConverter(std::string);
   ///< Set Properties::binspectrum_converter_
 # endif
 };
