@@ -60,7 +60,7 @@ class Gyoto::Astrobj::PolishDoughnut
 
  // Data : 
  // -----
-private:
+protected:
   /**
    * \brief KerrBL metric
    *
@@ -77,14 +77,19 @@ private:
  double r_cusp_; ///< Cusp radius in geometrical units. Tied to PolishDoughnut::lambda_.
  double r_centre_; ///< Central radius in geometrical units. Tied to PolishDoughnut::lambda_.
  double DeltaWm1_; ///< 1./(W_centre_ - W_surface_);
- double temperature_ratio_; ///< Central ion/electron temperature ratio
  double central_density_; ///< Central density in kg/L (same as g cm^-3)
+ /*
+   WARNING! so far (jan. 2014) central_density_ is density_central
+   in standard Polish doughnut model, but it is
+   density_central*c2+pressure_central in Komissarov model
+  */
  double centraltemp_over_virial_; ///< T<SUB>center</SUB>/T<SUB>virial</SUB>
  double beta_; ///< P<SUB>magn</SUB>/P<SUB>gas</SUB>
- int use_specific_impact_ ;///< Use PolishDoughnut::Impact_() or Standard::Impact()
  double aa_; ///< PolishDoughnut::gg_ spin, cached when setting PolishDoughnut::lambda_
  double aa2_; ///< aa_<SUP>2</SUP>
  size_t spectral_oversampling_;///< Oversampling used in integrateEmission()
+ bool komissarov_; ///< 1 if Komissarov model is integrated
+ bool angle_averaged_; ///< 1 if Komissarov model should be angle averaged
 
  // Constructors - Destructor
  // -------------------------
@@ -108,25 +113,25 @@ public:
  double getL0() const; ///< Get PolishDoughnut::l0_
  // void   setL0(double l0); set by lambda_
 
- double getLambda() const; ///< Get PolishDoughnut::lambda_
- void   setLambda(double lambda); ///< Set PolishDoughnut::lambda_
+ double lambda() const; ///< Get PolishDoughnut::lambda_
+ void   lambda(double lambda); ///< Set PolishDoughnut::lambda_
 
- double getTemperatureRatio() const; ///< Get PolishDoughnut::temperature_ratio_
- void   setTemperatureRatio(double temp); ///< Set PolishDoughnut::temperature_ratio_
+ double centralDensity() const; ///< Get PolishDoughnut::central_density_
+ double centralDensity(std::string unit) const; ///< Get PolishDoughnut::central_density_ in specified unit
+ void   centralDensity(double density); ///< Set PolishDoughnut::central_density_
+ void   centralDensity(double density, std::string unit); ///< Set PolishDoughnut::central_density_ in specified unit
 
- double getCentralDensity() const; ///< Get PolishDoughnut::central_density_
- double getCentralDensity(std::string unit) const; ///< Get PolishDoughnut::central_density_ in specified unit
- void   setCentralDensity(double density); ///< Set PolishDoughnut::central_density_
- void   setCentralDensity(double density, std::string unit); ///< Set PolishDoughnut::central_density_ in specified unit
+ double centralTempOverVirial() const; ///< Get PolishDoughnut::centraltemp_over_virial_
+ void   centralTempOverVirial(double val); ///< Set PolishDoughnut::centraltemp_over_virial_
 
- double getCentralTempOverVirial() const; ///< Get PolishDoughnut::centraltemp_over_virial_
- void   setCentralTempOverVirial(double val); ///< Set PolishDoughnut::centraltemp_over_virial_
+ double beta() const; ///< Get PolishDoughnut::beta_
+ void   beta(double beta);///< Set PolishDoughnut::beta_
 
- double getBeta() const; ///< Get PolishDoughnut::beta_
- void   setBeta(double beta);///< Set PolishDoughnut::beta_
+ void   spectralOversampling(size_t); ///< Set PolishDoughnut::spectral_oversampling_
+ size_t spectralOversampling() const ; ///< Get PolishDoughnut::spectral_oversampling_
 
- void   setSpectralOversampling(size_t); ///< Set PolishDoughnut::spectral_oversampling_
- size_t getSpectralOversampling() const ; ///< Get PolishDoughnut::spectral_oversampling_
+ bool komissarov() const; ///< Get PolishDoughnut::komissarov_
+ void komissarov(bool komis); ///< Set PolishDoughnut::komissarov_
 
  // Read only members, depend on lambda
  double getWsurface() const; ///< Get PolishDoughnut::W_surface_
@@ -134,28 +139,14 @@ public:
  double getRcusp() const; ///< Get PolishDoughnut::r_cusp_
  double getRcentre() const; ///< Get PolishDoughnut::r_centre_
 
-
- virtual Gyoto::SmartPointer<Gyoto::Metric::Generic> getMetric() const;
- virtual void setMetric(Gyoto::SmartPointer<Gyoto::Metric::Generic>);
- void useSpecificImpact(int yes=1); ///< Set PolishDoughnut::use_specific_impact_
+ virtual Gyoto::SmartPointer<Gyoto::Metric::Generic> metric() const;
+ virtual void metric(Gyoto::SmartPointer<Gyoto::Metric::Generic>);
 
  // ASTROBJ API
  // -----------
- /**
-  * Depending on PolishDoughnut::use_specific_impact_. See useSpecificImpact().
-  */
  int Impact(Photon *ph, size_t index,
 			    Astrobj::Properties *data);
- ///< Call either PolishDoughnut::Impact() or Standard::Impact()
 
- /**
-  * This should not be better than Standard::Impact(). Call
-  * useSpecificImpact() or Set &lt;UseSpecifictImpact/&gt; in the XML
-  * file to use it.
-  */
- int Impact_(Photon *ph, size_t index,
-			    Astrobj::Properties *data);
- ///< A specific implementation of Generic::Impact()
  virtual double operator()(double const coord[4]) ;
 
  virtual int setParameter(std::string name,
@@ -196,6 +187,10 @@ protected:
 			double dsem, double coord_ph[8],
 			double coord_obj[8]=NULL) const ;
 
+  void emission_komissarov(double Inu[], double nu_em[], size_t nbnu,
+			double dsem, double coord_ph[8],
+			double coord_obj[8]=NULL) const ;
+
   double emissionBrems(double nu_em, double nu_crit, 
 		       double numax, double T_electron,
 		       double n_e, double n_j,
@@ -211,6 +206,18 @@ protected:
 		       double alpha1, double alpha2,
 		       double alpha3, double preff,
 		       int comptonorder) const;
+
+  double emissionSynchro_komissarov_direction(double Theta_elec, 
+					      double number_density,
+					      double nuem,
+					      double nuc, 
+					      double theta
+					      ) const;
+  double emissionSynchro_komissarov_averaged(double Theta_elec, 
+					     double number_density,
+					     double nuem,
+					     double nuc 
+					     ) const;
   ///< Synchrotron proxy for emission()
   double transmission(double nuem, double dsem, double coord_ph[8]) const ;
   double BBapprox(double nuem, double Te) const; ///< Approximated Black-Body function
@@ -218,9 +225,9 @@ protected:
   ///< Mahadevan 96 fit function
  // PURELY INTERNAL TO ASTROBJ
  // --------------------------
-private:
- double potential(double r, double theta) const;
- ///< Potential defining shape, used by operator()()
+ private:
+  double potential(double r, double theta) const;
+  ///< Potential defining shape, used by operator()()
 
  /**
   * \class Gyoto::Astrobj::PolishDoughnut::intersection_t
@@ -235,11 +242,12 @@ private:
   */
   class intersection_t : public Gyoto::Functor::Double_Double_const {
   public:
-    double aa_; ///< Metric spin
-    double aa2_;///< aa_<SUP>2</SUP>
-    double l0_; ///< Torus angular momentum
+    intersection_t(PolishDoughnut* parent);
+    PolishDoughnut * papa;
     virtual double operator() (double) const;
   };
+  friend class intersection_t;
+
  /**
   * \class Gyoto::Astrobj::PolishDoughnut::transcendental_t
   * \brief double transcendental(double) Functor class
