@@ -56,7 +56,7 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
   char* filename_; ///< Lorene output file name
   Star_rot * star_; ///< Pointer to underlying Lorene Star_rot instance 
   int integ_kind_;///< 1 if RotStar3_1::myrk4(), 0 if Metric::myrk4()
- 
+
  public:
 
   RotStar3_1(); ///< Constructor
@@ -65,15 +65,14 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
   virtual RotStar3_1* clone() const ;
            ///< Cloner (uses RotStar3_1(file, integ_kind))
 
-  void setFileName(char const *); ///< Set filename_
-  char const * getFileName() const; ///< Get filename_
+  void fileName(char const *); ///< Set filename_
+  char const * fileName() const; ///< Get filename_
 
-  void setIntegKind(int); ///< Set integ_kind_
-  int getIntegKind() const ; ///< Get integ_kind_
+  void integKind(int); ///< Set integ_kind_
+  int integKind() const ; ///< Get integ_kind_
 
   using Metric::Generic::myrk4;
 
-
   /**
    * \brief RK4 integrator
    *
@@ -88,14 +87,14 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
    *
    * Dispatches between Generic::myrk4_adaptive() and  myrk4_adaptive(const double coor[6], double lastnorm, double normref, double coornew[6], double cst[2], double& tdot_used, double h0, double& h1, double& hused) const depending on RotStar3_1::integ_kind_
    */
-  int myrk4_adaptive(Gyoto::Worldline* line, const double coord[8], double lastnorm, double normref, double coordnew[8], double h0, double& h1) const;
+  int myrk4_adaptive(Gyoto::Worldline* line, const double coord[8], double lastnorm, double normref, double coordnew[8], double h0, double& h1, double h1max) const;
 
   /**
    * \brief RK4 integrator (3+1)
    *
    * NB: we use the 6-coordinates, here, unlike the inherited function.
    */
-  int myrk4_adaptive(const double coor[6], double lastnorm, double normref, double coornew[6], double cst[2], double& tdot_used, double h0, double& h1, double& hused) const;
+  int myrk4_adaptive(const double coor[6], double lastnorm, double normref, double coornew[6], double cst[2], double& tdot_used, double h0, double& h1, double h1max, double& hused) const;
 
   /**
    * \brief F function such as dy/dtau=F(y,cst)
