@@ -53,9 +53,7 @@ namespace Gyoto{
  * In addition, Quantities may be specified (or the default Quantity
  * will be produced: generally Intensity). Not all Astrobj implement
  * all Quantities. The order in which Quantities are listed is not
- * relevant (it is not stored). A value of the integration step for
- * the Photon's trajectory can be specified in Delta. It will be used
- * as the initial step, which is adaptive. Possible Quantities:
+ * relevant (it is not stored). Possible Quantities:
  *
  * - Intensity: the intensity that reaches the object, integrated over
  *        the line-of-sight;
@@ -81,10 +79,17 @@ namespace Gyoto{
  * actual number of cores available on the machine usually leads to a
  * decrease in performance.
  *
- * Thus a fully populated Scenery XML looks like that:
- * \code
- * <?xml version="1.0" encoding="UTF-8" standalone="no"?>
- * <Scenery>
+ * Finally, Scenery accepts a number of numerical tuning parameters
+ * that are passed directly to the underlying photons (actually, the
+ * Scenery object holds a Photon instance which stores many
+ * parameters, including the Metric and Astrobj):
+ * Adaptive/NonAdaptive, Delta, MinimumTime, MaxIter, PrimaryOnly.
+ *
+ * Thus a fully populated Scenery XML looks like that (the values are
+ * examples, they are not necessary the default nor the best or even
+ * good values):
+ * \code <?xml version="1.0" encoding="UTF-8"
+ * standalone="no"?> <Scenery>
  *
  *  <Metric kind = "MetricKind">
  *    <MetricProperties/>
@@ -100,10 +105,49 @@ namespace Gyoto{
  *
  *  <Quantities> Spectrum Intensity ...</Quantities>
  *
- *  <Delta> 1. </Delta>
- *
  *  <NThreads> 2 </NThreads>  
  *
+ *  Next come the numerical tuning parameters:
+ *  Integration step, initial in case of adaptive, reset for
+ *  for each ray being traced:
+ *  <Delta unit="geometrical"> 1 </Delta>
+ *
+ *  Adaptive or NonAdaptive:
+ *  <Adaptive/>
+ *
+ *  The integrator to use for integrating the photons:
+ *  <Integrator>runge_kutta_fehlberg78</Integrator>
+ *  The "Legacy" integrator is coded in
+ *  Metric::Generic::myrk4_adaptive(), may be re-implemented in othe
+ *  metrics, and therefore takes its tuning parameters in the Metric
+ *  section. The other integrators (runge_kutta_fehlberg78,
+ *  runge_kutta_cash_karp54, runge_kutta_dopri5,
+ *  runge_kutta_cash_karp54_classic) accept the following tuning
+ *  parameters, directly in the Scenery section:
+ *
+ *  Absolute and relative tolerance for the adaptive step:
+ *  <AbsTol>1e-11</AbsTol>
+ *  <RelTol>1e-11</RelTol>
+ *  Normally, you should not need to tune the other three. If you need
+ *  to, try using a higher order integrator:
+ *  maximum integration step:
+ *  <DeltaMax> 100 </DeltaMax>
+ *  delta_max/R where R is the current distance to the origin:
+ *  <DeltaMaxOverR> 0.1 </DeltaMaxOverR>
+ *  minimum step:
+ *  <DeltaMin>1e-20</DeltaMin>
+ *
+ *  A few safe-guards to avoid infinite loops:
+ *
+ *  Maximum number of iterations for each ray:
+ *  <Maxiter> 1000000 </Maxiter>
+ *
+ *  Minimum date a photon may reach backwards in time:
+ *  <MinimumTime unit="yr">25e3</MinimumTime>
+ *
+ *  This one is an experimental, poorly specified feature:
+ *  <!--PrimaryOnly/-->
+ *
  * </Scenery>
  * \endcode
  */
@@ -114,22 +158,12 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
   // Data : 
   // -----
  protected:
-  /**
-   * The Metric, or stage, for this scenery.
-   */
-  SmartPointer<Metric::Generic> gg_;
 
   /**
    * Screen, the camera for this scenery.
    */
   SmartPointer<Screen> screen_;
 
-  /**
-   * The astrophysical emitting light in this scenery... the actor.
-   */
-  SmartPointer<Astrobj::Generic> obj_;
-
-  bool   adaptive_; ///< Whether integration should use adaptive delta
 
   /**
    * Default integration step for the photons
@@ -146,18 +180,11 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
   Gyoto::Quantity_t quantities_;
 
   /**
-   * Used internally to not always reallocate memory when operator() is called.
+   * Used internally to not always reallocate memory when operator()
+   * is called and to store all the parameters which affect the
+   * integration, except delta_.
    */
-  Gyoto::Photon ph_; ///< Cached Photon.
-
-  /**
-   * Computation does not go back before tmin_. Default is -DBL_MAX. tmin_ is
-   * always expressed in geometrical units, it is essentially a tuning
-   * parameter for the ray-tracing process. tmin should be chosen to
-   * always be longer than the distance between the screen and the
-   * object.
-   */
-  double tmin_; ///< Time limit for the integration (geometrical units)
+  Gyoto::Photon ph_; ///< Template Photon.
 
   /**
    * When compiled with libpthread, Scenery::rayTrace() may compute
@@ -198,26 +225,30 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
  public:
   // Accessors
   // ---------
-  SmartPointer<Metric::Generic> getMetric(); ///< Get Scenery::gg_
+  SmartPointer<Metric::Generic> metric() const; ///< Get ph_.Worldline::metric_
   /**
    * The provided Metric will also be atached to the Screen and the Astrobj.
    */
-  void setMetric(SmartPointer<Metric::Generic>);  ///< Set Scenery::gg_
-  SmartPointer<Screen> getScreen(); ///< Get Scenery::screen_
+  void metric(SmartPointer<Metric::Generic>);  ///< Set Scenery::gg_
+  SmartPointer<Screen> screen() const; ///< Get Scenery::screen_
 
   /**
    * The Metric attached to the Scenery will be attached to the Screen
    */
-  void setScreen(SmartPointer<Screen>);///< Set Scenery::screen_
-  SmartPointer<Astrobj::Generic> getAstrobj(); ///< Get Scenery::obj_
+  void screen(SmartPointer<Screen>);///< Set Scenery::screen_
+  SmartPointer<Astrobj::Generic> astrobj() const; ///< Get ph_.obj_
   /**
    * The Metric attached to the Scenery will be attached to the Astrobj
    */
-  void setAstrobj(SmartPointer<Astrobj::Generic>); ///< Set Scenery::obj_
-  double getDelta() const ; ///< Get default step in geometrical units
-  double getDelta(const std::string &unit) const ;  ///< Get default step in specified units
-  void setDelta(double); ///< set default step in geometrical units
-  void setDelta(double, const std::string &unit);   ///< set default step in specified units
+  void astrobj(SmartPointer<Astrobj::Generic>); ///< Set ph_.obj_
+
+
+  SmartPointer<Photon> clonePhoton() const; ///< Clone the internal Photon
+
+  double delta() const ; ///< Get default step in geometrical units
+  double delta(const std::string &unit) const ;  ///< Get default step in specified units
+  void delta(double); ///< set default step in geometrical units
+  void delta(double, const std::string &unit);   ///< set default step in specified units
 
   /// Set Scenery::quantities_
   /**
@@ -244,30 +275,63 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    */
   size_t getScalarQuantitiesCount() const ;
 
-  /// Get Scenery::tmin_
-  double getTmin() const ;
-  /// Get Scenery::tmin_ in specified unit
-  double getTmin(const std::string &unit) const ;
-  /// Set Scenery::tmin_
-  void setTmin(double);
-  /// Set Scenery::tmin_ in specified unit
-  void setTmin(double, const std::string &unit);
+  /// Get ph_.tmin_
+  double tMin() const ;
+  /// Get ph_.tmin_ in specified unit
+  double tMin(const std::string &unit) const ;
+  /// Set ph_.tmin_
+  void tMin(double);
+  /// Set ph_.tmin_ in specified unit
+  void tMin(double, const std::string &unit);
 
-  void adaptive (bool mode) ; ///< Set Scenery::adaptive_
-  bool adaptive () const ; ///< Get Scenery::adaptive_
+  void adaptive (bool mode) ; ///< Set ph_.adaptive_
+  bool adaptive () const ; ///< Get ph_.adaptive_
 
-  void maxiter (size_t miter) ; ///< Set Scenery::maxiter_
-  size_t maxiter () const ; ///< Get Scenery::maxiter_
+  /// Passed to #ph_
+  void integrator(std::string type);
+  /// Passed to #ph_
+  std::string integrator() const;
 
-  void setNThreads(size_t); ///< Set nthreads_;
-  size_t getNThreads() const ; ///< Get nthreads_;
+  /// Passed to #ph_
+  double deltaMin() const;
+  /// Passed to #ph_
+  void deltaMin(double h1);
+
+  /// Passed to #ph_
+  double deltaMax() const;
+
+  /// Passed to #ph_
+  void deltaMax(double h1);
+
+  /// Passed to #ph_
+  double deltaMaxOverR() const;
+  /// Passed to #ph_
+  void deltaMaxOverR(double t);
+
+  /// Passed to #ph_
+  void absTol(double);
+  /// Passed to #ph_
+  double absTol()const;
+  /// Passed to #ph_
+  void relTol(double);
+  /// Passed to #ph_
+  double relTol()const;
+
+  void secondary (bool sec) ; ///< Set ph_.secondary_
+  bool secondary () const ; ///< Get ph_.secondary_
+
+  void maxiter (size_t miter) ; ///< Set ph_.maxiter_
+  size_t maxiter () const ; ///< Get ph_.maxiter_
+
+  void nThreads(size_t); ///< Set nthreads_;
+  size_t nThreads() const ; ///< Get nthreads_;
 
   /// Set Scenery::intensity_converter_
-  void setIntensityConverter(std::string unit);
+  void intensityConverter(std::string unit);
   /// Set Scenery::spectrum_converter_
-  void setSpectrumConverter(std::string unit);
+  void spectrumConverter(std::string unit);
   /// Set Scenery::binspectrum_converter_
-  void setBinSpectrumConverter(std::string unit);
+  void binSpectrumConverter(std::string unit);
 
   /// Copy converters to Astrobj::Properties instance
   /**
