@@ -1,5 +1,5 @@
 /*
-    Copyright 2011 Thibaut Paumard
+    Copyright 2011-2014 Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -30,6 +30,9 @@
 #include "GyotoScenery.h"
 #include "GyotoScreen.h"
 
+#include <cstring>
+#include <sstream>
+
 #define YGYOTO_TYPE_LEN 21
 #define YGYOTO_STD_CATCH catch(Gyoto::Error e) \
   { y_error(e.get_message().c_str()); }
@@ -131,9 +134,11 @@ void ygyoto_Spectrometer_generic_eval
 // Keywords processed by ygyoto_Metric_generic_eval
 #define YGYOTO_METRIC_GENERIC_KW "prime2tdot",				\
     "nullifycoord", "kind", "setparameter", "scalarprod",		\
-    "mass", "unitlength", "circularvelocity", "xmlwrite", "clone"
+    "mass", "deltamin", "deltamax", "keplerian",					\
+    "unitlength", "circularvelocity",					\
+    "christoffel", "xmlwrite", "clone"
 // Number of those keywords
-#define YGYOTO_METRIC_GENERIC_KW_N 10
+#define YGYOTO_METRIC_GENERIC_KW_N 14
 
 // Keywords processed by ygyoto_Astrobj_generic_eval
 #define YGYOTO_ASTROBJ_GENERIC_KW "metric", "rmax", "opticallythin",	\
@@ -198,26 +203,36 @@ void ygyoto_Spectrometer_generic_eval
 #define YGYOTO_CAT1(X, Y) X##Y
 #define YGYOTO_CAT(X, Y) YGYOTO_CAT1(X,Y)
 
-#define YGYOTO_CONSTRUCTOR_INIT1(BASENAME, BASECLASS, DERIVEDCLASS)	\
+#ifdef GYOTO_USE_XERCES
+# define YGYOTO_CONSTRUCTOR_INIT2_XML(BASENAME, GETTER)			\
+  char * fname = ygets_q(argc-1);					\
+  OBJ = ypush_##BASENAME();						\
+  GYOTO_DEBUG_EXPR(OBJ);						\
+  * OBJ = Gyoto::Factory(fname).GETTER();				\
+  GYOTO_DEBUG << "Swapping object for filename\n";	                \
+  yarg_swap(0, argc);			\
+  GYOTO_DEBUG << "Dropping filename from stack\n";			\
+  yarg_drop(1);							\
+  GYOTO_DEBUG << "Dropped filename from stack\n";			\
+  --argc;
+#else
+# define YGYOTO_CONSTRUCTOR_INIT2_XML(BASENAME, GETTER)	\
+  y_error("this gyoto was built without XML support");
+#endif
+
+
+#define YGYOTO_CONSTRUCTOR_INIT2(BASENAME, BASECLASS, DERIVEDCLASS, GETTER)\
   Gyoto::SmartPointer<BASECLASS> *OBJ = NULL;				\
   if (yarg_##BASENAME(argc-1)) {					\
     OBJ = yget_##BASENAME(--argc);					\
     GYOTO_DEBUG_EXPR(OBJ);						\
   } else if (yarg_string(argc-1)) {					\
-    char * fname = ygets_q(argc-1);					\
-    OBJ = ypush_##BASENAME();						\
-    GYOTO_DEBUG_EXPR(OBJ);						\
-    * OBJ = Gyoto::Factory(fname).get##BASENAME();			\
-    GYOTO_DEBUG << "Swapping object for filename\n";			\
-    yarg_swap(0, argc);							\
-    GYOTO_DEBUG << "Dropping filename from stack\n";			\
-    yarg_drop(1);							\
-    GYOTO_DEBUG << "Dropped filename from stack\n";			\
-    --argc;								\
+    YGYOTO_CONSTRUCTOR_INIT2_XML(BASENAME, GETTER);			\
   } else {								\
     OBJ = ypush_##BASENAME();						\
     GYOTO_DEBUG_EXPR(OBJ);						\
     *OBJ = new DERIVEDCLASS();						\
+    GYOTO_DEBUG << "object created" << endl;				\
     for (int arg=0; arg<argc; ++arg)					\
       yarg_swap(arg, arg+1);						\
   }									\
@@ -225,7 +240,8 @@ void ygyoto_Spectrometer_generic_eval
     yarg_drop(1);							\
     --argc;								\
   }
-
+#define YGYOTO_CONSTRUCTOR_INIT1(BASENAME, BASECLASS, DERIVEDCLASS)    \
+  YGYOTO_CONSTRUCTOR_INIT2(BASENAME, BASECLASS, DERIVEDCLASS, get##BASENAME)
 #define YGYOTO_CONSTRUCTOR_INIT(BASE, KIND) \
   YGYOTO_CONSTRUCTOR_INIT1(BASE, Gyoto::BASE::Generic, Gyoto::BASE::KIND)
 
@@ -253,7 +269,9 @@ void ygyoto_Spectrometer_generic_eval
   int k=-1;								\
   char const * rmsg="Cannot set return value more than once";		\
   char const * pmsg="Cannot use positional argument more than once";	\
-  char * unit=NULL;
+  char * unit=NULL;							\
+  /* make sure we use the variables at least once */			\
+  if (OBJ && rmsg && pmsg && unit) {}
 
 
 #define YGYOTO_WORKER_GETSET_VECTOR(MEMBER, N)			  \
@@ -285,6 +303,16 @@ void ygyoto_Spectrometer_generic_eval
 	(*OBJ) -> set##MEMBER (ygets_d(iarg), unit?unit:"");	   \
     }
 
+#define YGYOTO_WORKER_GETSET_DOUBLE2_UNIT(MEMBER)			   \
+  if ((iarg=kiargs[++k])>=0) {					   \
+      iarg+=*rvset;						   \
+      if (yarg_nil(iarg)) {					   \
+	if ((*rvset)++) y_error("Only one return value possible"); \
+	ypush_double((*OBJ) -> MEMBER (unit?unit:""));	   \
+      } else							   \
+	(*OBJ) -> MEMBER (ygets_d(iarg), unit?unit:"");	   \
+    }
+
 #define YGYOTO_WORKER_GETSET_DOUBLE(MEMBER)			   \
   if ((iarg=kiargs[++k])>=0) {					   \
       iarg+=*rvset;						   \
@@ -315,6 +343,16 @@ void ygyoto_Spectrometer_generic_eval
 	(*OBJ) -> MEMBER (ygets_d(iarg));			   \
     }
 
+#define YGYOTO_WORKER_GETSET_STRING2(MEMBER)			   \
+  if ((iarg=kiargs[++k])>=0) {					   \
+      iarg+=*rvset;						   \
+      if (yarg_nil(iarg)) {					   \
+	if ((*rvset)++) y_error("Only one return value possible"); \
+	*ypush_q(0) = p_strcpy((*OBJ) -> MEMBER () . c_str() );	   \
+      } else							   \
+	(*OBJ) -> MEMBER (std::string(ygets_q(iarg)));		   \
+    }
+
 #define YGYOTO_WORKER_GETSET_LONG(MEMBER)			   \
   if ((iarg=kiargs[++k])>=0) {					   \
       iarg+=*rvset;						   \
@@ -348,6 +386,19 @@ void ygyoto_Spectrometer_generic_eval
     }								   \
   }
 
+#define YGYOTO_WORKER_GETSET_OBJECT2(MEMBER,YMEMBER)			   \
+  if ((iarg=kiargs[++k])>=0) {					   \
+    iarg+=*rvset;						   \
+    if (yarg_nil(iarg)) {					   \
+      GYOTO_DEBUG << "pushing " #MEMBER << std::endl;		   \
+      if ((*rvset)++) y_error("Only one return value possible");   \
+      *ypush_##YMEMBER () = (*OBJ) -> MEMBER ();		   \
+    } else {							   \
+      GYOTO_DEBUG << "setting " #MEMBER << std::endl;		   \
+      (*OBJ) -> MEMBER (*yget_##YMEMBER (kiargs[k]));	   \
+    }								   \
+  }
+
 #define YGYOTO_WORKER_SET_UNIT		 \
   if ((iarg=kiargs[++k])>=0) {		 \
     iarg+=*rvset;			 \
@@ -398,112 +449,6 @@ void ygyoto_Spectrometer_generic_eval
   ygyoto_##BASE##_generic_eval(YGYOTO_CAT(OBJ,_), \
 			       kiargs+k+1, piargs, rvset, paUsed, unit);
 
-
-/*
-  The following are to declare a new base (such as Metric or Astrobj)
-  or independent (such as Photon) class. Seldom used in a Gyoto
-  extension.
- */
-#ifdef GYOTO_USE_XERCES
-# define YGYOTO_PRINT_YUSEROBJ(NAME)				\
-  void gyoto_##NAME##_print(void *obj) {				\
-    string rest="", sub="";					\
-    size_t pos=0, len=0;					\
-    try {rest = Factory(((gyoto_##NAME*)obj)->smptr).format();}	\
-    YGYOTO_STD_CATCH;						\
-    while (len=rest.length())  {				\
-      sub=rest.substr(0, pos=rest.find_first_of("\n",0));	\
-      rest=rest.substr(pos+1, len-1);				\
-      y_print( sub.c_str(),1 );					\
-    }								\
-  }
-#else
-# define YGYOTO_PRINT_YUSEROBJ(NAME)    \
-  void gyoto_##NAME##_print(void *obj) { \
-    y_print("GYOTO " #NAME,0);	       \
-  }
-#endif 
-#define YGYOTO_YUSEROBJ(NAME, CLASS)					\
-  extern "C" {								\
-    typedef struct gyoto_##NAME {					\
-      Gyoto::SmartPointer<CLASS> smptr;					\
-    } gyoto_##NAME;							\
-    void gyoto_##NAME##_free(void *obj) {				\
-      if (((gyoto_##NAME*)obj)->smptr) {				\
-	((gyoto_##NAME*)obj)->smptr=NULL;				\
-      } else printf("null pointer\n");					\
-    }									\
-    YGYOTO_PRINT_YUSEROBJ(NAME)						\
-    void gyoto_##NAME##_eval(void *obj, int argc);			\
-    void gyoto_##NAME##_extract(void *obj, char *member) {		\
-      long idxo = yget_global("__gyoto_obj", 0);			\
-      long idxr = yget_global("__gyoto_res", 0);			\
-      *ypush_##NAME()= ((gyoto_##NAME*)obj)->smptr;			\
-      yput_global(idxo, 0);						\
-      yarg_drop(1);							\
-      long dims[Y_DIMSIZE]={1,1};					\
-      string stmt = "eq_nocopy, __gyoto_res, __gyoto_obj(";		\
-      stmt.append(member).append("=);");				\
-      *ypush_q(dims)=p_strcpy(stmt.c_str());				\
-      yexec_include(0, 1);						\
-      yarg_drop(1);							\
-      ypush_global(idxr);						\
-    }									\
-    static y_userobj_t gyoto_##NAME##_obj =				\
-    {const_cast<char*>("gyoto_" #NAME),					\
-     &gyoto_##NAME##_free,						\
-     &gyoto_##NAME##_print,						\
-     &gyoto_##NAME##_eval,						\
-     &gyoto_##NAME##_extract,						\
-     0};								\
-  }									\
-  Gyoto::SmartPointer<CLASS>* yget_##NAME(int iarg) {			\
-    return &(((gyoto_##NAME*)yget_obj(iarg, &gyoto_##NAME##_obj))->smptr); \
-  }									\
-  Gyoto::SmartPointer<CLASS>* ypush_##NAME() {				\
-  return &(((gyoto_##NAME*)ypush_obj(&gyoto_##NAME##_obj,		\
-				     sizeof(gyoto_##NAME)))->smptr);	\
-  }									\
-  int yarg_##NAME(int iarg) {						\
-    return yget_obj(iarg,0)==gyoto_##NAME##_obj.type_name;		\
-  }									\
-  extern "C" {								\
-    void Y_is_gyoto_##NAME(int argc)					\
-    {									\
-      ypush_long(yarg_##NAME(0));					\
-    }									\
-  }
-#define YGYOTO_BASE_CONSTRUCTOR(BASE)					\
-  extern "C" {								\
-  void Y_gyoto_##BASE(int argc)	{					\
-    Gyoto::SmartPointer<Gyoto::BASE::Generic> *OBJ = NULL;		\
-    if (yarg_##BASE(argc-1)) {						\
-      OBJ = yget_##BASE(argc);						\
-    } else {								\
-      if (!yarg_string(argc-1))						\
-        y_error("Cannot allocate object of virtual class " #BASE);	\
-      char * fname = ygets_q(argc-1);					\
-      OBJ = ypush_##BASE();						\
-      Gyoto::BASE::Subcontractor_t * sub =				\
-	Gyoto::BASE::getSubcontractor(fname, 1);			\
-      if (sub) {							\
-	GYOTO_DEBUG << "found a subcontractor for \"" << fname		\
-		    << "\", calling it now\n";				\
-	*OBJ = (*sub)(NULL);						\
-      } else {								\
-	GYOTO_DEBUG << "found no subcontractor for \"" << fname		\
-		    << "\", calling Factory now\n";			\
-	*OBJ = Factory(fname).get##BASE();				\
-      }									\
-      yarg_swap(0, argc);						\
-      yarg_drop(1);							\
-    }									\
-    --argc;								\
-    gyoto_##BASE##_eval(OBJ, argc);					\
-  }									\
-}
-
-
 /*
 
   The following are needed to export the ABI to other plug-ins.
